package com.fr.plugin;

import com.fr.decision.authority.data.User;
import com.fr.decision.fun.impl.AbstractGlobalRequestFilterProvider;
import com.fr.decision.webservice.utils.DecisionServiceConstants;
import com.fr.decision.webservice.v10.login.LoginService;
import com.fr.decision.webservice.v10.user.UserService;
import com.fr.log.FineLoggerFactory;
import com.fr.plugin.dao.MyDao_IPRange;
import com.fr.plugin.dao.MyDao_UserToken;
import com.fr.plugin.entity.MyIPRange;
import com.fr.plugin.entity.MyUserToken;
import com.fr.plugin.transform.FunctionRecorder;
import com.fr.stable.StringUtils;
import com.fr.stable.db.accessor.DBAccessor;
import com.fr.stable.fun.Authorize;
import com.fr.stable.query.QueryFactory;
import com.fr.stable.query.condition.QueryCondition;
import com.fr.stable.query.restriction.RestrictionFactory;
import com.fr.swift.source.core.MD5Utils;
import com.fr.web.utils.WebUtils;

import javax.servlet.FilterChain;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Date;
import java.util.List;

@Authorize(callSignKey = MyFunctionConstants.PLUGIN_ID)
@FunctionRecorder
public class MyGlobalRequestFilter extends AbstractGlobalRequestFilterProvider
{
    @Override
    public String filterName()
    {
        return "login_test1";
    }

    @Override
    public String[] urlPatterns()
    {
        return new String[]{"/decision/*"};
    }

    @Override
    public void doFilter(HttpServletRequest req, HttpServletResponse res, FilterChain fc)
    {
        String parameterToken = req.getParameter("token");
        String userName = req.getParameter("userName");
        String t = req.getParameter("t");
        String userTokenKey;

        //获取客户端IP
        String LocalIPAddress = req.getLocalAddr();

        try
        {
            if (StringUtils.isNotBlank(parameterToken))
            {
                //login_token不为空时查询数据库表，检查是否有相应userToken
                DBAccessor dbAccessor = MyDecisionDBAccessProvider.getDbAccessor();

                List<MyUserToken> myUserTokenList = dbAccessor.runQueryAction(daoContext ->
                {
                    QueryCondition queryCondition = QueryFactory.create();
                    queryCondition.addRestriction(RestrictionFactory.eq("userName", userName));
                    return daoContext.getDAO(MyDao_UserToken.class).find(queryCondition);
                });

                userTokenKey = myUserTokenList.get(0).getUserToken();

                List<MyIPRange> myUserIPList = dbAccessor.runQueryAction(daoContext ->
                {
                    QueryCondition queryCondition = QueryFactory.create();
                    queryCondition.addRestriction(RestrictionFactory.and(
                            RestrictionFactory.eq("id", myUserTokenList.get(0).getId()),
                            RestrictionFactory.eq("IPAddress", LocalIPAddress)));
                    return daoContext.getDAO(MyDao_IPRange.class).find(queryCondition);
                });

                if (myUserIPList.size() > 0)
                {
                    if(!LoginFR(userName, t, parameterToken, userTokenKey, req, res))
                    {
                        FineLoggerFactory.getLogger().error("登陆失败: 请联系管理员~~~");
                        WebUtils.printAsString(res, "登陆出错~");
                    }
                }
                else
                {
                    FineLoggerFactory.getLogger().error("IP不在允许范围~~~");
                    WebUtils.printAsString(res, "IP不在允许范围~~~");
                }
            }
            fc.doFilter(req,res);
        }
        catch (Exception e)
        {
            e.printStackTrace();
            FineLoggerFactory.getLogger().error("登陆失败: 请联系管理员~~~");
        }
    }

    private boolean LoginFR(String username,String t,String token, String tokenKey, HttpServletRequest req, HttpServletResponse res)
    {


        try{
            long startTime1 = new Date().getTime() - 30*1000;
            long endTime1 = new Date().getTime() + 30*1000;
            long lt = Long.parseLong(t);
            String md5 = MD5Utils.getMD5String(new String[]{username, t, tokenKey});
            User user = UserService.getInstance().getUserByUserName(username);

            if (user == null)
            {
                WebUtils.printAsString(res,"登陆失败，用户 " + username + " 不存在~");
                return  false;
            }

            if(!token.equals(md5))
            {
                FineLoggerFactory.getLogger().error("Token无效~~~");
                WebUtils.printAsString(res, "Token无效~~~");
                return  false;
            }

            if (startTime1 > lt || endTime1 < lt)
            {
                FineLoggerFactory.getLogger().error("Token过期，请重新获取Token~~~");
                WebUtils.printAsString(res, "Token过期，请重新获取Token~~~");
                return  false;
            }

            String tokenResult = LoginService.getInstance().login(req, res,username);
            req.setAttribute(DecisionServiceConstants.FINE_AUTH_TOKEN_NAME, tokenResult);
            return  true;
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        return  false;
    }
}
